<?php

include_once('Database.php');
include_once('Application.php');

class System extends Application
{
  var $Database;
  var $Config;
  var $ModuleManager;
  var $PathItems;
  var $Menu;
  var $DoNotShowPage;
  var $OnPageNotFound;

  function __construct()
  {
    $this->Config = array();
    $this->Menu = array();
    $this->RSSChannels = array();
    $this->DoNotShowPage = false;
    $this->OnPageNotFound = array();
  }

  function Init()
  {
    global $Config, $LocaleManager;

    $this->Config = $Config;
    $this->Database = new Database();
    $this->Database->Connect($this->Config['Database']['Host'],
      $this->Config['Database']['User'], $this->Config['Database']['Password'],
      $this->Config['Database']['Database']);
    $this->Database->charset($this->Config['Database']['Charset']);
    $this->Database->ShowSQLQuery = $this->Config['Web']['ShowSQLQuery'];
    $this->Database->ShowSQLError = $this->Config['Web']['ShowSQLError'];
    $this->Database->LogSQLQuery = $this->Config['Web']['LogSQLQuery'];
    $this->ModuleManager = new AppModuleManager();

    $this->PathItems = ProcessURL();
    // Initialize locale
    $LocaleManager = new LocaleManager($this);
    $LocaleManager->Dir = dirname(dirname(__FILE__)).'/locale';
    $LocaleManager->Available = array(
      'cs' => array('Code' => 'cs', 'Title' => 'Česky'),
      'en' => array('Code' => 'en', 'Title' => 'English'),
    );
    $Code = $Config['Web']['Locale'];
    if (isset($_SESSION))
      if(array_key_exists('Locale', $_SESSION)) $Code = $_SESSION['Locale'];
    if (isset($_GET))
      if(array_key_exists('locale', $_GET)) $Code = $_GET['locale'];

    //echo(phpinfo());
    if(!array_key_exists($Code, $LocaleManager->Available)) $Code = 'en';
    $_SESSION['Locale'] = $Code;
    $LocaleManager->LoadLocale($Code);

    $this->Menu = array
    (
   /*     array(
            'Title' => T('Files'),
            'Hint' => 'Stahování různých pomocných souborů a programů',
            'Link' => $this->Link('/download.php'),
            'Permission' => LICENCE_ANONYMOUS,
            'Icon' => '',
        ),
        */
        array(
            'Title' => T('Instructions'),
            'Hint' => 'Informace k překladu hry',
            'Link' => $this->Link('/info.php'),
            'Permission' => LICENCE_ANONYMOUS,
            'Icon' => '',
        ),
        array(
            'Title' => T('Data source'),
            'Hint' => 'Informace o překladových skupinách',
            'Link' => $this->Link('/TranslationList.php?action=grouplist'),
            'Permission' => LICENCE_ANONYMOUS,
            'Icon' => '',
        ),
        array(
            'Title' => T('Presentation'),
            'Hint' => 'Prezentace a motivace překladu',
            'Link' => $this->Link('/promotion.php'),
            'Permission' => LICENCE_ANONYMOUS,
            'Icon' => '',
        ),
        array(
            'Title' => T('IRC chat'),
            'Hint' => 'IRC chat pro překladatele',
            'Link' => 'http://embed.mibbit.com/?server=game.zdechov.net%3A6667&amp;channel=%23wowpreklad&amp;forcePrompt=true&amp;charset=utf-8',
            'Permission' => LICENCE_ANONYMOUS,
            'Icon' => '',
        ),
        array(
            'Title' => T('Administration'),
            'Hint' => 'Volby pro správu',
            'Link' => $this->Link('/admin/'),
            'Permission' => LICENCE_ADMIN,
            'Icon' => '',
        ),
    );
  }

  function Run()
  {
    global $System, $ScriptStartTime, $TranslationTree, $User, $StopAfterUpdateManager,
      $UpdateManager, $Config, $DatabaseRevision;

    $ScriptStartTime = GetMicrotime();

    if(GetRemoteAddress() != '') session_start();

    if(!isset($Config)) die('Systém není nainstalován. Pokračujte v instalaci <a href="admin/install.php">zde</a>.');
    date_default_timezone_set($Config['Web']['Timezone']);

    $this->Init();

    // Check database persistence structure
    $UpdateManager = new UpdateManager();
    $UpdateManager->Database = $this->Database;
    $UpdateManager->Revision = $DatabaseRevision;
    if(!$UpdateManager->IsInstalled()) die('Systém vyžaduje instalaci databáze.');
    if(!$UpdateManager->IsUpToDate()) die('Systém vyžaduje aktualizaci databáze.');

    // SQL injection hack protection
    foreach($_POST as $Index => $Item)
    {
      if(is_array($_POST[$Index]))
        foreach($_POST[$Index] as $Index2 => $Item2) $_POST[$Index][$Index2] = addslashes($Item2);
      else $_POST[$Index] = addslashes($_POST[$Index]);
    }
    foreach($_GET as $Index => $Item) $_GET[$Index] = addslashes($_GET[$Index]);

    // TODO: Global initialized variable should be removed
    $TranslationTree = GetTranslationTree();

    // Initialize application modules
    $this->ModuleManager->RegisterModule(new ModuleError($System));
    $this->ModuleManager->Modules['Error']->ShowError = $Config['Web']['ShowPHPError'];
    $this->ModuleManager->RegisterModule(new ModuleLog($System));
    $this->ModuleManager->RegisterModule(new ModuleUser($System));
    $this->ModuleManager->RegisterModule(new ModuleAoWoW($System));
    $this->ModuleManager->RegisterModule(new ModuleReferrer($System));
    $this->ModuleManager->Modules['Referrer']->Excludes[] = $System->Config['Web']['Host'];
    $this->ModuleManager->RegisterModule(new ModuleTeam($System));
    $this->ModuleManager->RegisterModule(new ModuleDictionary($System));
    $this->ModuleManager->RegisterModule(new ModuleTranslation($System));
    $this->ModuleManager->RegisterModule(new ModuleImport($System));
    $this->ModuleManager->RegisterModule(new ModuleExport($System));
    $this->ModuleManager->RegisterModule(new ModuleServer($System));
    $this->ModuleManager->RegisterModule(new ModuleClientVersion($System));
    $this->ModuleManager->RegisterModule(new ModuleShoutBox($System));
    $this->ModuleManager->RegisterModule(new ModuleNews($System));
    $this->ModuleManager->RegisterModule(new ModuleWiki($System));
    $this->ModuleManager->RegisterModule(new ModuleSearch($System));
    $this->ModuleManager->RegisterModule(new ModuleFrontPage($System));
    $this->ModuleManager->RegisterModule(new ModuleDownload($System));
    $this->ModuleManager->RegisterModule(new ModuleForum($System));
    $this->ModuleManager->RegisterModule(new ModuleRedirection($System));
    $this->ModuleManager->StartAll();

    $this->BaseView = new BaseView($this);
    if($this->DoNotShowPage == false)
    {
      $this->ShowPage();
    }
  }

  function GetMicrotime()
  {
    list($Usec, $Sec) = explode(' ', microtime());
    return ((float)$Usec + (float)$Sec);
  }

  function Link($Target)
  {
    return($this->Config['Web']['BaseURL'].$Target);
  }

  function RegisterPage($Path, $Handler)
  {
    if(is_array($Path))
    {
      $Page = &$this->Pages;
      $LastKey = array_pop($Path);
      foreach($Path as $PathItem)
      {
        $Page = &$Page[$PathItem];
      }
      if(!is_array($Page)) $Page = array('' => $Page);
      $Page[$LastKey] = $Handler;
    } else $this->Pages[$Path] = $Handler;
  }

  function RegisterMenuItem($MenuItem, $Pos = NULL)
  {
    if(is_null($Pos)) $this->Menu[] = $MenuItem;
      else {
        array_splice($this->Menu, $Pos, 0, array($MenuItem));
      }
  }

  function SearchPage($PathItems, $Pages)
  {
    if(count($PathItems) == 0) $PathItems = array('');
    $PathItem = $PathItems[0];
    if(array_key_exists($PathItem, $Pages))
    {
      if(is_array($Pages[$PathItem]))
      {
        array_shift($PathItems);
        return($this->SearchPage($PathItems, $Pages[$PathItem]));
      } else
      {
        if(count($PathItems) == 1) return($Pages[$PathItem]);
          else return(''); // Unexpected subpages
      }
    } else return('');
  }

  function PageNotFound()
  {
    // Send correct HTTP status code to signal unknown page
    if(array_key_exists('SERVER_PROTOCOL', $_SERVER))
      Header($_SERVER['SERVER_PROTOCOL'].' 404 Not Found');
    if(array_key_exists('HTTP_REFERER', $_SERVER)) $Referer = ' Referer: '.$_SERVER['HTTP_REFERER'];
      else $Referer = '';
    if(isset($this->ModuleManager->Modules['Log']))
      $this->ModuleManager->Modules['Log']->WriteLog('Stránka "'.
        implode('/', $this->PathItems).'" nenalezena'.$Referer, LOG_TYPE_PAGE_NOT_FOUND);
    return(ShowMessage(sprintf(T('Page "%s" not found'), implode('/', $this->PathItems)), MESSAGE_CRITICAL));
  }

  function ShowPage()
  {
    $Output = '';
    /* @var $Page Page */
    $ClassName = $this->SearchPage($this->PathItems, $this->Pages);
    if(($ClassName != '') and (class_exists($ClassName)))
    {
      $Page = new $ClassName($this);
      $Output = $Page->GetOutput();
      $this->BaseView->Title = $Page->Title;
      if($Page->RawPage == false) $Output = $this->BaseView->ShowPage($Output);
    } else {
      $Output2 = '';
      if((count($this->OnPageNotFound) == 2)
  and method_exists($this->OnPageNotFound[0], $this->OnPageNotFound[1]))
        $Output2 = call_user_func_array($this->OnPageNotFound, array());
     if($Output2 != '') $Output .= $this->BaseView->ShowPage($Output2);
      else {
        $Output = $this->PageNotFound();
        $this->BaseView->Title = T('Page not found');
        $Output = $this->BaseView->ShowPage($Output);
      }
    }
    echo($Output);
  }
}

class BaseView extends View
{
  var $Title;

  function ShowTopBar()
  {
    global $LocaleManager;

    $Output = '<div class="Menu">';
    if(!$this->System->User->Licence(LICENCE_USER))
      $Output .= '<div class="advert">'.$this->System->Config['Web']['Advertisement'].'</div>';
    $Output .= '<span class="MenuItem"></span><span class="MenuItem2">';
    if($this->System->User->Licence(LICENCE_USER))
    {
      //$DbResult = $System->Database->query('SELECT `Id`, `Name` FROM `Team` WHERE `Id`='.$this->System->User->Team);
      //$Team = $DbResult->fetch_assoc();
      //$Output .= ''<span class="MenuItem">Moje překlady: <a href="">Dokončené</a> <a href="">Rozpracované</a> <a href="">Exporty</a> Tým: <a href="">'.$Team['name'].'</a></span>';
      $Output .= $this->System->User->Name.' <a href="'.$this->System->Link('/?action=logout').'">'.T('Logout').'</a>'.
          ' <a href="'.$this->System->Link('/user.php?user='.$this->System->User->Id).'">'.T('My page').'</a>'.
          ' <a href="'.$this->System->Link('/Options.php').'">'.T('Options').'</a>'.
          ' <a title="Vámi přeložené texty" href="'.$this->System->Link('/TranslationList.php?user='.
          $this->System->User->Id.'&amp;group=0&amp;state=2&amp;text=&amp;entry=').'">'.T('Translated').'</a>'.
          ' <a title="Vaše rozpracované text" href="'.$this->System->Link('/TranslationList.php?user='.
          $this->System->User->Id.'&amp;group=0&amp;state=3&amp;text=&amp;entry=').'">'.T('Unfinished').'</a>'.
          ' <a title="Nikým nepřeložené texty" href="'.
          $this->System->Link('/TranslationList.php?user=0&amp;group=0&amp;state=1&amp;text=&amp;entry=').'">'.T('Untranslated').'</a>';
    } else
    {
      $Output .= '<a href="'.$this->System->Link('/login/').'">'.T('Login').'</a>&nbsp;'.
          '<a href="'.$this->System->Link('/registrace.php').'">'.T('Registration').'</a>';
    }
    //$Output .= ' <form action="?setlocale" method="get">';
    $Output .= ' <select onchange="window.location=\'?locale=\'+this.value">';
    foreach($LocaleManager->Available as $Locale)
    {
      if($Locale['Code'] == $LocaleManager->CurrentLocale->Texts->Code) $Selected = ' selected="selected"';
        else $Selected = '';
      $Output .= '<option title="" value="'.$Locale['Code'].'"'.$Selected.' onchange="this.form.submit()">'.$Locale['Title'].'</option>';
    }
    $Output .= '</select><noscript><span><input type="submit" value="Submit"/></span></noscript>';
    //$Output .= '</form>';
    $Output .= '</span></div>';
    return($Output);
  }

  function ShowLoginBox()
  {
    $Output = '';
    if($this->System->User->Licence(LICENCE_USER))
    {
      // $Output .= 'Jste přihlášen jako: <b>'.$tUser->Id.'</b> <a href="index.php?Logout">Odhlásit</a>';
    } else
    {
      $Output .= '<strong>'.T('Login').':</strong>
      <form action="" method="post">
      <table>
      <tr>
      <td><input type="text" name="LoginUser" size="13" /></td>
      </tr>
      <tr>
      <td><input type="password" name="LoginPass" size="13" /></td>
      </tr>
      <tr>
      <th><input type="submit" value="'.T('Do login').'" /></th>
      </tr>
      </table>
      </form>';
    }
    return($Output);
  }

  function ShowSearchBox()
  {
    $Output = '<strong>'.T('Search').':</strong>'.
      '<form action="'.$this->System->Link('/search/').'" method="get"><div>'.
      '<table>'.
      '<tr>'.
      '<td><input type="text" name="text" size="13" /></td>'.
      '</tr>'.
      '<tr>'.
      '<th><input type="submit" value="'.T('Do search').'" /></th>'.
      '</tr>'.
      '</table></div>'.
      '</form>';
    return($Output);
  }

  function ShowMainMenu()
  {
    $Output = '<strong>'.T('Menu').':</strong>'.
      '<div class="verticalmenu"><ul>';
    foreach($this->System->Menu as $MenuItem)
      if($this->System->User->Licence($MenuItem['Permission']))
      {
        if(isset($MenuItem['Click'])) $OnClick = ' onclick="'.$MenuItem['Click'].'"';
        else $OnClick = '';
        if($MenuItem['Icon'] != '') $Icon = '<img src="'.$this->System->Link('/images/menu/'.$MenuItem['Icon']).'"/>';
        else $Icon = '';
        $Output .= '<li>'.$Icon.'<a class="verticalmenua" title="'.$MenuItem['Hint'].'" href="'.
          $MenuItem['Link'].'"'.$OnClick.'>'.$MenuItem['Title'].'</a></li>';
      }
      $Output .= '</ul></div>';
      return($Output);
  }

  function ShowTranslatedMenu()
  {
    global $TranslationTree;

    $Output = '<strong>'.T('Translate groups').':</strong><br /><div id="TranslationMenu">';
    $DbResult = $this->System->Database->select('Group', '`Id`, `Name`', '1 ORDER BY `Name`');
    while($Group = $DbResult->fetch_assoc())
    {
      $Output .= '<div id="menuitem-group'.$Group['Id'].'" onmouseover="show(\'group'.$Group['Id'].'\')" onmouseout="hide(\'group'.$Group['Id'].'\')">'.
          '<a href="'.$this->System->Link('/TranslationList.php?group='.$Group['Id'].'&amp;action=filter').'">'.str_replace(' ','&nbsp;',$Group['Name']).'</a></div>'.
          '<div id="group'.$Group['Id'].'" class="hidden-menu-item" onmouseover="show(\'group'.$Group['Id'].'\')" onmouseout="hide(\'group'.$Group['Id'].'\')">';
      $Output .= '&nbsp;<a title="Zde můžete začít překládat" href="'.$this->System->Link('/TranslationList.php?group='.$Group['Id'].'&amp;state=1&amp;user=0&amp;entry=&amp;text=').'">Nepřeložené</a><br />'.
          '&nbsp;<a title="Přeložené texty, můžete zde hlasovat, nebo opravovat překlady" href="'.$this->System->Link('/TranslationList.php?group='.$Group['Id'].'&amp;state=2&amp;user=0&amp;entry=&amp;text=').'">Přeložené</a><br />';
      if($this->System->User->Licence(LICENCE_USER))
      {
        $Output .= '&nbsp;<a title="Nedokončené překlady" href="'.$this->System->Link('/TranslationList.php?group='.$Group['Id'].'&amp;state=3').'">Rozepsané</a><br />
        &nbsp;<a title="Všechny překlady, které jste přeložil" href="'.$this->System->Link('/TranslationList.php?group='.$Group['Id'].'&amp;state=1&amp;user='.$this->System->User->Id).'&amp;entry=&amp;text=">Vlastní</a><br />';
      }
      $Output .= '&nbsp;<a title="Sestavit speciální filtr" href="'.$this->System->Link('/TranslationList.php?group='.$Group['Id'].'&amp;action=filter').'">Filtr</a><br />';
      $Output .= '</div>';
    }
    $Output .= '</div>';
    return($Output);
  }

  function ShowHeader()
  {
    $Output = ''.
    '<!DOCTYPE html>'.
    '<html>'.
    '<head>'.
    '<meta http-equiv="content-type" content="text/html; charset='.$this->System->Config['Web']['Charset'].'" />'.
    '<meta name="keywords" content="wow, quest, questy, questů, preklad, mangos, překlad, překládání, přeložený, přeložení, čeština, world of warcraft, open source, free, addon" />'.
    '<meta name="description" content="'.$this->System->Config['Web']['Description'].'" />'.
    '<meta name="robots" content="all" />'.
    '<link rel="stylesheet" href="'.$this->System->Link('/style/style.css').'" type="text/css" media="all" />'.
    '<script type="text/javascript" src="'.$this->System->Link('/style/global.js').'"></script>'.
    '<link rel="shortcut icon" href="'.$this->System->Link('/images/favicon.ico').'" />';
    $Output .= $this->System->ModuleManager->Modules['News']->ShowRSSHeader();
    $Title = $this->System->Config['Web']['Title'];
    if($this->Title != '') $Title = $this->Title.' - '.$Title;
    $Output .= '<title>'.$Title.'</title>'.
    '</head><body>';

    $Output .= $this->ShowTopBar();
    $Output .= '<table class="page"><tr><td class="menu">';
    $Output .= $this->ShowMainMenu();
    $Output .= $this->System->ModuleManager->Modules['User']->ShowOnlineList();
    $Output .= '<br />';
    $Output .= $this->ShowSearchBox();

    $Output .= '</td><td id="border-left"></td><td class="content">';
    return($Output);
  }

  function ShowFooter()
  {
    global $ScriptStartTime, $Revision, $ReleaseTime;

    $ScriptGenerateDuration = round(GetMicrotime() - $ScriptStartTime, 2);

    $Output = '</td>'.
      '<td class="menu2">';
    $Output .= $this->ShowTranslatedMenu();
    $Output .= '</td>'.
      '</tr><tr>'.
      '<td colspan="4" class="page-bottom">'.T('Version').': '.$Revision.' ('.HumanDate($ReleaseTime).')'.
      ' &nbsp; <a href="http://svn.zdechov.net/trac/wowpreklad/browser/trunk">'.T('Source code').'</a> &nbsp; '.
      '<a href="http://svn.zdechov.net/trac/wowpreklad/log/trunk?verbose=on">'.T('Changelog').'</a> &nbsp; '.
      $this->System->Config['Web']['WebCounter'];

    $Output .= '</td></tr>';
    if($this->System->Config['Web']['ShowRuntimeInfo'] == true)
      $Output .= '<tr><td colspan="3" style="text-align: center;">'.T('Generating duration').': '.
    $ScriptGenerateDuration.' s / '.ini_get('max_execution_time').' s &nbsp;&nbsp; '.T('Used memory').': '.
    HumanSize(memory_get_peak_usage(FALSE)).' / '.ini_get('memory_limit').'B &nbsp;&nbsp; <a href="http://validator.w3.org/check?uri='.
    htmlentities('http://'.$_SERVER['HTTP_HOST'].GetRequestURI().'?'.$_SERVER['QUERY_STRING']).'">HTML validator</a></td></tr>';
    $Output .= '</table>'.
      '</body>'.
      '</html>';
    return($Output);
  }

  function ShowPage($Content)
  {
    $Output = $this->ShowHeader().$Content.$this->ShowFooter();
    if($this->System->Config['Web']['FormatOutput'])
      $Output = $this->FormatOutput($Output);
    return($Output);
  }

  function FormatOutput($s)
  {
    $out = '';
    $nn = 0;
    $n = 0;
    while($s != '')
    {
      $start = strpos($s, '<');
      $end = strpos($s, '>');
      if($start != 0)
      {
        $end = $start - 1;
        $start = 0;
      }
      $line = trim(substr($s, $start, $end + 1));
      if(strlen($line) > 0)
        if($line[0] == '<')
        {
          if($s[$start + 1] == '/')
          {
            $n = $n - 2;
            $nn = $n;
          } else
          {
            if(strpos($line, ' ')) $cmd = substr($line, 1, strpos($line, ' ') - 1);
            else $cmd = substr($line, 1, strlen($line) - 2);
            //echo('['.$cmd.']');
            if(strpos($s, '</'.$cmd.'>')) $n = $n + 2;
          }
        }// else $line = '['.$line.']';
        //if($line != '') echo(htmlspecialchars(str_repeat(' ',$nn).$line."\n"));
        if($nn < 0) $nn = 0;
        if($line != '') $out .= (str_repeat(' ', $nn).$line."\n");
        $s = substr($s, $end + 1, strlen($s));
        $nn = $n;
    }
    return($out);
  }
}
